<?php

namespace Drupal\sendgrid_integration_simplenews_reports\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;

/**
 * Provides a date range form for SendGrid statistics.
 */
class DateRangeForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sendgrid_simplenews_date_range_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?NodeInterface $node = NULL, $start_date = NULL, $end_date = NULL) {
    // Set default values if not provided.
    if (empty($start_date)) {
      $start_date = date('Y-m-d', strtotime('-1 month'));
    }
    if (empty($end_date)) {
      $end_date = date('Y-m-d');
    }

    // Store node for use in submit handler.
    $form_state->set('node', $node);

    $form['date_range'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['date-range-form']],
    ];

    // Determine the node creation date for the minimum start date.
    $min_start_date = NULL;
    if ($node instanceof \Drupal\node\NodeInterface && !empty($node->created->value)) {
      $min_start_date = date('Y-m-d', $node->created->value);
    }

    $form['date_range']['start_date'] = [
      '#type' => 'date',
      '#title' => $this->t('Start Date'),
      '#default_value' => $start_date,
      '#required' => TRUE,
      '#min' => $min_start_date,
    ];

    $form['date_range']['end_date'] = [
      '#type' => 'date',
      '#title' => $this->t('End Date'),
      '#default_value' => $end_date,
      '#required' => TRUE,
      '#max' => date('Y-m-d'),
    ];

    $form['date_range']['actions'] = [
      '#type' => 'actions',    ];

    $form['date_range']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Apply'),    ];

    $form['#attached']['library'][] = 'sendgrid_integration_simplenews_reports/date_range_form';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $start_date = $form_state->getValue('start_date');
    $end_date = $form_state->getValue('end_date');

    // Validate that end date is not before start date.
    if (strtotime($end_date) < strtotime($start_date)) {
      $form_state->setErrorByName('end_date', $this->t('End date cannot be before start date.'));
    }

    // Validate that start date is not before node creation date (if node is set).
    $node = $form_state->get('node');
    if ($node instanceof \Drupal\node\NodeInterface && !empty($node->created->value)) {
      $node_created = date('Y-m-d', $node->created->value);
      if (strtotime($start_date) < strtotime($node_created)) {
        $form_state->setErrorByName('start_date', $this->t('Start date cannot be before the newsletter creation date (@date).', ['@date' => $node_created]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $node = $form_state->get('node');

    // Redirect to the same page with query parameters.
    $form_state->setRedirect('sendgrid_integration_simplenews_reports.node_statistics', [
      'node' => $node->id(),
    ], [
      'query' => [
        'start_date' => $form_state->getValue('start_date'),
        'end_date' => $form_state->getValue('end_date'),
      ],
    ]);
  }

}
